/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* callbacks.c: this file is part of runlevel-admin, a ximian-setup-tool frontend 
 * for run level services administration.
 * 
 * Copyright (C) 2002 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors: Carlos Garnacho <garparr@teleline.es>.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "xst.h"
#include "e-table.h"
#include "callbacks.h"
#include "runlevel-druid.h"

extern XstTool *tool;

int reply;

static void callbacks_button_set_complexity(XstDialog *main_dialog){
	switch(xst_dialog_get_complexity(main_dialog)){
		case XST_DIALOG_ADVANCED:
			gtk_widget_show(xst_dialog_get_widget(main_dialog,"rc0"));
			gtk_widget_show(xst_dialog_get_widget(main_dialog,"rc1"));
			gtk_widget_show(xst_dialog_get_widget(main_dialog,"rc2"));
			gtk_widget_show(xst_dialog_get_widget(main_dialog,"rc3"));
			gtk_widget_show(xst_dialog_get_widget(main_dialog,"rc4"));
			gtk_widget_show(xst_dialog_get_widget(main_dialog,"rc6"));
			break;
		case XST_DIALOG_BASIC:
			gtk_widget_hide(xst_dialog_get_widget(main_dialog,"rc0"));
			gtk_widget_hide(xst_dialog_get_widget(main_dialog,"rc1"));
			gtk_widget_hide(xst_dialog_get_widget(main_dialog,"rc2"));
			gtk_widget_hide(xst_dialog_get_widget(main_dialog,"rc3"));
			gtk_widget_hide(xst_dialog_get_widget(main_dialog,"rc4"));
			gtk_widget_hide(xst_dialog_get_widget(main_dialog,"rc6"));
			break;
		default:
			g_warning("Unknown complexity");
			break;
	}
}
	
void on_main_dialog_update_complexity(GtkWidget *main_dialog, gpointer data){
	XstDialogComplexity complexity;
	complexity=XST_DIALOG(main_dialog)->complexity;
	callbacks_button_set_complexity(XST_DIALOG(main_dialog));
}

gboolean callbacks_conf_read_failed_hook (XstTool *tool, XstReportLine *rline, gpointer data){
	GtkWidget *dialog;
	gchar *txt;

	txt = g_strdup_printf (_("The file ``%s'' is missing or could not be read:\nThe configuration will show empty."), rline->argv[0]);
        
	dialog = gnome_error_dialog_parented (txt, GTK_WINDOW (tool->main_dialog));
	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	
	g_free (txt);

	return TRUE;
}

void reply_cb (gint val, gpointer data){
        reply = val;
        gtk_main_quit ();
}


void on_service_add_clicked(GtkWidget *button, gpointer data){
	RunlevelDruid *druid;
	gint runlevel=gtk_notebook_current_page(GTK_NOTEBOOK(xst_dialog_get_widget(tool->main_dialog,"runlevel_admin")));
	druid=runlevel_druid_new(runlevel);
	runlevel_druid_show(druid);
}

void on_service_delete_clicked(GtkWidget *button, gpointer data){
	gchar *service,*buf;
	gint runlevel=gtk_notebook_current_page(GTK_NOTEBOOK(xst_dialog_get_widget(tool->main_dialog,"runlevel_admin")));
	GtkWidget *dialog;
	xmlNodePtr node;
	
	g_return_if_fail(xst_tool_get_access(tool));
	g_return_if_fail(node=get_selected_node(runlevel));
	service=xst_xml_get_child_content(node,"name");
	buf=g_strdup_printf(_("Are you sure you want to delete %s from runlevel %i?"),service,runlevel);
	dialog=gnome_question_dialog_parented(buf,reply_cb,NULL,GTK_WINDOW(tool->main_dialog));
	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
	g_free(service);
	g_free(buf);
	if (reply){
		return;
	}
	table_delete (runlevel);
	xst_dialog_modify (tool->main_dialog);
	on_buttons_set_sensitive (FALSE);
}
void on_buttons_set_sensitive(gboolean state){
	gint level;
	gchar *del_button;
	if (xst_tool_get_access(tool)){
		level=gtk_notebook_current_page(GTK_NOTEBOOK(xst_dialog_get_widget(tool->main_dialog,"runlevel_admin")));
		del_button=g_strdup_printf("rc%i_delete_button",level);
		gtk_widget_set_sensitive(xst_dialog_get_widget(tool->main_dialog,del_button),state);
	}
}

