/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* e-table.c: this file is part of runlevel-admin, a ximian-setup-tool frontend 
 * for run level services administration.
 * 
 * Copyright (C) 2002 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors: Carlos Garnacho <garparr@teleline.es>.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "xst.h"
#include <gal/e-table/e-table-scrolled.h>
#include <gal/e-table/e-table-simple.h>
#include <gal/e-table/e-cell-text.h>

#include "e-table.h"
#include "callbacks.h"

#define RUNLEVEL_TABLE_SPEC "runlevel.etspec"

extern XstTool *tool;

GtkWidget *runlevel_table[7];
GArray *runlevel_array[7];

const gchar *runlevel_state = "\
<ETableState> \
  <column source=\"0\"/> \
  <column source=\"1\"/> \
  <column source=\"2\"/> \
  <grouping> \
    <leaf column=\"2\" descending=\"true\"/> \
  </grouping> \
</ETableState>";

int table_col_count(ETableModel *etc,void *data){
	return 3;
}

int table_row_count(ETableModel *etc,void *data,int level){
	if (runlevel_array[level]){
		return runlevel_array[level]->len;
	}
	return 0;
}

int rc0_table_row_count(ETableModel *etc,void *data){ return table_row_count(etc,data,0); }
int rc1_table_row_count(ETableModel *etc,void *data){ return table_row_count(etc,data,1); }
int rc2_table_row_count(ETableModel *etc,void *data){ return table_row_count(etc,data,2); }
int rc3_table_row_count(ETableModel *etc,void *data){ return table_row_count(etc,data,3); }
int rc4_table_row_count(ETableModel *etc,void *data){ return table_row_count(etc,data,4); }
int rc5_table_row_count(ETableModel *etc,void *data){ return table_row_count(etc,data,5); }
int rc6_table_row_count(ETableModel *etc,void *data){ return table_row_count(etc,data,6); }

void* table_value_service(xmlNodePtr node){
	gchar *buf;
	g_return_val_if_fail(node!=NULL,NULL);
	buf=xst_xml_get_child_content(node,"name");
	return buf;
}

void* table_value_order(xmlNodePtr node){
	gchar *buf;
	g_return_val_if_fail(node!=NULL,NULL);
	buf=xst_xml_get_child_content(node,"priority");
	return buf;
}

void* table_value_sk(xmlNodePtr node){
	gchar *buf;
	g_return_val_if_fail(node!=NULL,NULL);
	buf=xst_xml_get_child_content(node,"action");
	return buf;
}

void* table_value_at(ETableModel *etc, int col, int row, void *data,int level){
	xmlNodePtr node;
	node=g_array_index(runlevel_array[level], xmlNodePtr, row);
	switch(col){
		case COL_SERVICE:
			return table_value_service(node);
			break;
		case COL_ORDER:
			return table_value_order(node);
			break;
		case COL_SK:
			return table_value_sk(node);
			break;
		default:
			return NULL;
	}
}

void* rc0_table_value_at(ETableModel *etc,int col,int row,void *data){ return table_value_at(etc,col,row,data,0); }
void* rc1_table_value_at(ETableModel *etc,int col,int row,void *data){ return table_value_at(etc,col,row,data,1); }
void* rc2_table_value_at(ETableModel *etc,int col,int row,void *data){ return table_value_at(etc,col,row,data,2); }
void* rc3_table_value_at(ETableModel *etc,int col,int row,void *data){ return table_value_at(etc,col,row,data,3); }
void* rc4_table_value_at(ETableModel *etc,int col,int row,void *data){ return table_value_at(etc,col,row,data,4); }
void* rc5_table_value_at(ETableModel *etc,int col,int row,void *data){ return table_value_at(etc,col,row,data,5); }
void* rc6_table_value_at(ETableModel *etc,int col,int row,void *data){ return table_value_at(etc,col,row,data,6); }

void table_set_value_at (ETableModel *etc, int col, int row, const void *val, void *data){
}

gboolean table_is_cell_editable (ETableModel *etc, int col, int row, void *data){
        return FALSE;
}

void *table_duplicate_value (ETableModel *etc, int col, const void *value, void *data){
        return g_strdup (value);
}

void table_free_value (ETableModel *etc, int col, void *value, void *data){
        g_free (value);
}

void *table_initialize_value (ETableModel *etc, int col, void *data){
        return g_strdup ("");
}

gboolean table_value_is_empty (ETableModel *etc, int col, const void *value, void *data){
        return !(value && *(char *)value);
}

char *table_value_to_string (ETableModel *etc, int col, const void *value, void *data){
        return g_strdup(value);
}

ETableExtras *create_extras (void){
        ETableExtras *extras;
        ECell *ec;
        extras = e_table_extras_new ();
        ec = e_cell_text_new (NULL, GTK_JUSTIFY_LEFT);
        e_table_extras_add_cell (extras, "centered_cell", ec);
        return extras;
}

void on_runlevel_table_cursor_change(ETable *table,gint row,gpointer data){
	on_buttons_set_sensitive(TRUE);
}

GtkWidget* table_create(char *widget,char *number){
	gint level;
	ETableModel *model;
	ETableExtras *extras;
	ETable *table;
	gchar *spec;
	void* row_count[]={
		&(rc0_table_row_count),
		&(rc1_table_row_count),
		&(rc2_table_row_count),
		&(rc3_table_row_count),
		&(rc4_table_row_count),
		&(rc5_table_row_count),
		&(rc6_table_row_count)
	};
	void* value_at[]={
		&(rc0_table_value_at),
		&(rc1_table_value_at),
		&(rc2_table_value_at),
		&(rc3_table_value_at),
		&(rc4_table_value_at),
		&(rc5_table_value_at),
		&(rc6_table_value_at)
	};
		
	level=number[0]-'0';
	
	if (runlevel_table[level]) return NULL;
	extras=create_extras();
	
	model= e_table_simple_new(table_col_count,
	                          row_count[level],
	                          NULL,
	                          value_at[level],
			          table_set_value_at,
			          table_is_cell_editable,
			          NULL,
			          NULL,
			          table_duplicate_value,
			          table_free_value,
			          table_initialize_value,
			          table_value_is_empty,
			          table_value_to_string,
			          NULL);
	spec=xst_conf_get_string(tool,"spec");
	if (!spec){
		spec=xst_ui_load_etspec(tool->etspecs_common_path, RUNLEVEL_TABLE_SPEC);
		if (!spec)
			g_error("create table: Couldn't create table.");
		xst_conf_set_string(tool,"spec",spec);
	}
	runlevel_table[level]=e_table_scrolled_new(E_TABLE_MODEL(model),extras,spec,runlevel_state);
	g_free(spec);
	g_return_val_if_fail(runlevel_table[level]!=NULL,NULL);
	
	table=e_table_scrolled_get_table(E_TABLE_SCROLLED(runlevel_table[level]));
	gtk_signal_connect(GTK_OBJECT(table),"cursor_change",on_runlevel_table_cursor_change,NULL);
	
	return runlevel_table[level];
}

xmlNodePtr get_selected_node(gint runlevel){
	ETable *table;
	gint row;
	
	table=e_table_scrolled_get_table(E_TABLE_SCROLLED(runlevel_table[runlevel]));
	row=e_table_get_cursor_row(table);
	g_return_val_if_fail(row>=0,NULL);
	return g_array_index(runlevel_array[runlevel],xmlNodePtr, row);
}

void table_populate(xmlNodePtr root){
	xmlNodePtr node,service;
	gint runlevel;
	ETable *table[7];
	
	g_return_if_fail(root!=NULL);
	for(node=xst_xml_element_find_first(root,"runlevel"),runlevel=0;node!=NULL;node=xst_xml_element_find_next(node,"runlevel"),runlevel++){
		table[runlevel]=e_table_scrolled_get_table(E_TABLE_SCROLLED(runlevel_table[runlevel]));
		runlevel_array[runlevel]=g_array_new(FALSE,FALSE,sizeof(xmlNodePtr));
		for (service=xst_xml_element_find_first(node,"service");service!=NULL;service=xst_xml_element_find_next(service,"service")){
			g_array_prepend_val(runlevel_array[runlevel],service);
		}
		e_table_model_changed(table[runlevel]->model);
	}
}

void table_add(gint runlevel,gchar *service,gint priority,gchar *action){
	xmlNodePtr node,root,runl;
	ETable *table;
	gchar *prio;
	gint row,runlevel_count;
	
	prio=g_strdup_printf("%i",priority);
	table=e_table_scrolled_get_table(E_TABLE_SCROLLED(runlevel_table[runlevel]));
	root=xst_xml_doc_get_root(tool->config);
	for(runl=xst_xml_element_find_first(root,"runlevel"),runlevel_count=0;runlevel_count!=runlevel;runl=xst_xml_element_find_next(runl,"runlevel"),runlevel_count++);	
	node=xst_xml_element_add(runl,"service");
	xst_xml_element_add_with_content(node,"name",service);
	xst_xml_element_add_with_content(node,"priority",prio);
	xst_xml_element_add_with_content(node,"action",action);
	g_array_append_val(runlevel_array[runlevel],node);
	row=runlevel_array[runlevel]->len-1;
	e_table_model_row_inserted(table->model,row);
}

void table_delete(gint runlevel){
	ETable *table;
	gint row;
	xmlNodePtr node;
	
	table=e_table_scrolled_get_table(E_TABLE_SCROLLED(runlevel_table[runlevel]));
	row=e_table_get_cursor_row(table);
	node=g_array_index(runlevel_array[runlevel],xmlNodePtr,row);
	
	xst_xml_element_destroy(node);
	g_array_remove_index(runlevel_array[runlevel],row);
	e_table_model_row_deleted(table->model,row);
}
