/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* runlevel-druid.c: this file is part of runlevel-admin, a ximian-setup-tool frontend 
 * for run level services administration.
 * 
 * Copyright (C) 2002 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors: Carlos Garnacho <garparr@teleline.es>.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <gal/e-table/e-table-scrolled.h>
#include <gal/e-table/e-table-simple.h>

#include "runlevel-druid.h"
#include "service_e-table.h"
#include "e-table.h"
#include "xst.h"

extern GArray *service_array;
extern XstTool *tool;

void runlevel_druid_update_buttons(GnomeDruid *druid,gboolean active){
	g_return_if_fail(druid!=NULL);
	gnome_druid_set_buttons_sensitive(druid,TRUE,active,TRUE);
}

void runlevel_druid_exit(GtkWidget *widget,gpointer data){
	RunlevelDruid *druid=data;
	gtk_widget_destroy(druid->window);
	g_free(druid->glade);
	g_free(druid);
}

void runlevel_druid_unset_next_button(GnomeDruidPage *page,GnomeDruid *druid, gpointer data){
	RunlevelDruid *r_druid=data;
	ETable *table;
	GtkWidget *widget;
	widget=glade_xml_get_widget(r_druid->glade,"service_table");
	table=e_table_scrolled_get_table(E_TABLE_SCROLLED(widget));
	if (e_table_get_cursor_row(table)==-1){
		runlevel_druid_update_buttons(druid,FALSE);
	}else{
		runlevel_druid_update_buttons(druid,TRUE);
	}
}

void runlevel_druid_set_next_button(ETable *change,gint row,gpointer data){
	RunlevelDruid *r_druid=data;
	GtkWidget *druid=glade_xml_get_widget(r_druid->glade,"druid");
	runlevel_druid_update_buttons(GNOME_DRUID(druid),TRUE);
}

void runlevel_druid_save_changes(GnomeDruidPage *page,GnomeDruid *druid, gpointer data){
	RunlevelDruid *r_druid=data;
	gchar *service;
	gint priority;
	gchar *action;

	GtkWidget *widget;
	ETable *table;
	xmlNodePtr node;
	gint row;
	
	widget=glade_xml_get_widget(r_druid->glade,"service_table");
	table=e_table_scrolled_get_table(E_TABLE_SCROLLED(widget));
	row=e_table_get_cursor_row(table);
	node=g_array_index(service_array, xmlNodePtr,row);
	service=xst_xml_element_get_content(node);
	widget=glade_xml_get_widget(r_druid->glade,"priority");
	priority=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
	widget=glade_xml_get_widget(r_druid->glade,"start");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))){
		action=g_strdup("start");
	}else{
		action=g_strdup("kill");
	}
	table_add(r_druid->runlevel,service,priority,action);
//	runlevel_xml_add(runlevel,service,priority,start);
	
        xst_dialog_modify (tool->main_dialog);
	runlevel_druid_exit(GTK_WIDGET(r_druid->window),r_druid);
}

void runlevel_druid_connect_signals (RunlevelDruid *druid){
	XstDialogSignal signals[]={
		{"druid","cancel",runlevel_druid_exit},
		{"page1","prepare",runlevel_druid_unset_next_button},
		{"page3","finish",runlevel_druid_save_changes},
		{NULL}
	};
	gint i;
	GtkWidget *widget;
	ETable *table;

	g_return_if_fail (druid != NULL);
	for (i=0; signals[i].widget; i++){
		widget = glade_xml_get_widget(druid->glade,signals[i].widget);
		if (widget){
			gtk_signal_connect (GTK_OBJECT (widget), signals[i].signal_name, GTK_SIGNAL_FUNC (signals[i].func),druid);
		}
	}
	widget=glade_xml_get_widget(druid->glade,"service_table");
	table=e_table_scrolled_get_table(E_TABLE_SCROLLED(widget));
        gtk_signal_connect(GTK_OBJECT(table),"cursor_change",runlevel_druid_set_next_button,druid);
}

RunlevelDruid* runlevel_druid_new(gint runlevel){
	RunlevelDruid* druid;

	druid=g_new0(RunlevelDruid,1);
	druid->glade=glade_xml_new(tool->glade_path,"window2");
	g_return_val_if_fail(druid->glade!=NULL,NULL);
	druid->window=glade_xml_get_widget(druid->glade,"window2");
	druid->runlevel=runlevel;
        gtk_window_set_title (GTK_WINDOW (druid->window), _("Runlevel Wizard"));
	gtk_window_set_modal(GTK_WINDOW(druid->window),TRUE);
	runlevel_druid_connect_signals(druid);
	return druid;	
}

void runlevel_druid_show(RunlevelDruid *druid){
	gtk_widget_show(druid->window);
}
