/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/* service_e-table.c: this file is part of runlevel-admin, a ximian-setup-tool frontend 
 * for run level services administration.
 * 
 * Copyright (C) 2002 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors: Carlos Garnacho <garparr@teleline.es>.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "xst.h"
#include <gal/e-table/e-table-scrolled.h>
#include <gal/e-table/e-table-simple.h>
#include <gal/e-table/e-cell-text.h>

#include "service_e-table.h"
#include "callbacks.h"

#define SERVICE_TABLE_SPEC "service.etspec"

extern XstTool *tool;

GtkWidget *service_table;
GArray *service_array;

const gchar *service_state = "\
<ETableState> \
  <column source=\"0\"/> \
  <grouping> \
  </grouping> \
</ETableState>";

int service_table_col_count(ETableModel *etc,void *data){
	return 1;
}

int service_table_row_count(ETableModel *etc,void *data){
	if (service_array){
		return service_array->len;
	}
	return 0;
}

void* service_table_value_service(xmlNodePtr node){
	gchar *buf;
	g_return_val_if_fail(node!=NULL,NULL);
	buf=xst_xml_element_get_content(node);
	return buf;
}

void* service_table_value_at(ETableModel *etc, int col, int row, void *data){
	xmlNodePtr node;
	node=g_array_index(service_array, xmlNodePtr, row);
	switch(col){
		case 0:
			return service_table_value_service(node);
			break;
		default:
			return NULL;
	}
}

void service_table_set_value_at (ETableModel *etc, int col, int row, const void *val, void *data){
}

gboolean service_table_is_cell_editable (ETableModel *etc, int col, int row, void *data){
        return FALSE;
}

void *service_table_duplicate_value (ETableModel *etc, int col, const void *value, void *data){
        return g_strdup (value);
}

void service_table_free_value (ETableModel *etc, int col, void *value, void *data){
        g_free (value);
}

void *service_table_initialize_value (ETableModel *etc, int col, void *data){
        return g_strdup ("");
}

gboolean service_table_value_is_empty (ETableModel *etc, int col, const void *value, void *data){
        return !(value && *(char *)value);
}

char *service_table_value_to_string (ETableModel *etc, int col, const void *value, void *data){
        return g_strdup(value);
}

ETableExtras *service_create_extras (void){
        ETableExtras *extras;
        ECell *ec;
        extras = e_table_extras_new ();
        ec = e_cell_text_new (NULL, GTK_JUSTIFY_LEFT);
        e_table_extras_add_cell (extras, "centered_cell", ec);
        return extras;
}

GtkWidget* service_table_create(void){
	ETableModel *model;
	ETableExtras *extras;
//	ETable *table;
	gchar *spec;
//	if (service_table) return service_table;
	extras=service_create_extras();
	model= e_table_simple_new(service_table_col_count,
	                          service_table_row_count,
	                          NULL,
	                          service_table_value_at,
			          service_table_set_value_at,
			          service_table_is_cell_editable,
			          NULL,
			          NULL,
			          service_table_duplicate_value,
			          service_table_free_value,
			          service_table_initialize_value,
			          service_table_value_is_empty,
			          service_table_value_to_string,
			          NULL);
	spec=xst_ui_load_etspec(tool->etspecs_common_path, SERVICE_TABLE_SPEC);
	if (!spec){
		g_error("create table: Couldn't create table.");
		return NULL;
	}
	service_table=e_table_scrolled_new(E_TABLE_MODEL(model),extras,spec,service_state);
	g_free(spec);
	g_return_val_if_fail(service_table!=NULL,NULL);
	return service_table;
}

void service_table_populate(xmlNodePtr root){
	xmlNodePtr node,service;
	ETable *table;
	
	g_return_if_fail(root!=NULL);
	
	node=xst_xml_element_find_first(root,"services");
	table=e_table_scrolled_get_table(E_TABLE_SCROLLED(service_table));
	service_array=g_array_new(FALSE,FALSE,sizeof(xmlNodePtr));
	for (service=xst_xml_element_find_first(node,"service");service!=NULL;service=xst_xml_element_find_next(service,"service")){
		g_array_prepend_val(service_array,service);
	}
	e_table_model_changed(table->model);
}

